#include "objLoader.h"
#include<QFile>
#include<QTextStream>
#include<iostream>
#include <sstream>
#include <fstream>
#include <glm/fwd.hpp>

/* Internal structure */
struct _ObjMeshFaceIndex{
    int pos_index[3];
    int tex_index[3];
    int nor_index[3];
};

#define TOKEN_VERTEX_POS "v"
#define TOKEN_VERTEX_NOR "vn"
#define TOKEN_VERTEX_TEX "vt"
#define TOKEN_FACE "f"

//TODO: multiple meshes
using namespace ObjLoader;
ObjMesh ObjLoader::LoadObjMesh(const char *filename){
    ObjMesh myMesh;

    std::vector<glm::vec3>           positions;
    std::vector<glm::vec2>           texcoords;
    std::vector<glm::vec3>           normals;
    std::vector<_ObjMeshFaceIndex>  faces;
    /**
         * Load file, parse it
         * Lines beginning with:
         * '#'  are comments can be ignored
         * 'v'  are vertices positions (3 floats that can be positive or negative)
         * 'vt' are vertices texcoords (2 floats that can be positive or negative)
         * 'vn' are vertices normals   (3 floats that can be positive or negative)
         * 'f'  are faces, 3 values that contain 3 values which are separated by / and <space>
         */

    {
        //std::ifstream filestream;
        //filestream.open(filename.c_str());
        QFile fajl(QString::fromStdString(filename));
        if(!fajl.open(QFile::ReadOnly)){
            std::cerr<<"ne morem odpreti: "<<filename;
            throw "err";
        }

        //std::string line_stream;	// No longer depending on char arrays thanks to: Dale Weiler
        QTextStream t( &fajl );
        t.setCodec("UTF-8");
        while ( !t.atEnd() ) {

            //while(fajl.readLine line_stream)){
            //while(std::getline(filestream, line_stream)){

            std::istringstream str_stream(t.readLine().toStdString());
            std::string type_str;
            str_stream >> type_str;
            if(type_str == TOKEN_VERTEX_POS){
                glm::vec3 pos;
                str_stream >> pos.x >> pos.y >> pos.z;
                positions.push_back(pos);
            }else if(type_str == TOKEN_VERTEX_TEX){
                glm::vec2 tex;
                str_stream >> tex.x >> tex.y;
                texcoords.push_back(tex);
            }else if(type_str == TOKEN_VERTEX_NOR){
                glm::vec3 nor;
                str_stream >> nor.x >> nor.y >> nor.z;
                normals.push_back(nor);
            }else if(type_str == TOKEN_FACE){
                _ObjMeshFaceIndex face_index;
                char interupt;
                for(int i = 0; i < 3; ++i){
                    face_index.pos_index[i]=face_index.nor_index[i]=face_index.tex_index[i]=0;

                    str_stream >> face_index.pos_index[i];
                    str_stream >> interupt;
                    if(interupt==' '){
                        continue;//brez normal in faceov
                    }else if(interupt=='/'){//prvi
                        str_stream >> interupt;
                        if(interupt=='/'){//drugi

                        }else{
                            str_stream.unget();
                            str_stream>>face_index.tex_index[i];
                            //beri tex
                            str_stream >> interupt;
                        }


                        if(interupt=='/' ){//drugi
                            str_stream>>face_index.nor_index[i];
                        }//ce je space gremo dalje

                    }else{
                        //TODO: error
                    }



                    //str_stream >> face_index.pos_index[i];
                }
                faces.push_back(face_index);
            }
        }
        // Explicit closing of the file
        //filestream.close();
        fajl.close();
    }

    for(size_t i = 0; i < faces.size(); ++i){
        ObjMeshFace face;
        _ObjMeshFaceIndex* faceFrom=&faces[i];

        for(size_t j = 0; j < 3; ++j){
            face.vertices[j].Position = positions[faceFrom->pos_index[j] - 1];
            if(texcoords.size()>faceFrom->tex_index[j] - 1){
                face.vertices[j].Tex    = texcoords[faceFrom->tex_index[j] - 1];
            }
            if(normals.size()>faceFrom->nor_index[j] - 1){
                face.vertices[j].Normal = normals[faceFrom->nor_index[j] - 1];
            }
        }
        myMesh.faces.push_back(face);
    }

    return myMesh;
}
